import matplotlib.pyplot as plt
from .color_print import print_color

def per_base_n_content(seqs):
    greater_20 = 0
    greater_10 = 0
    
    # Initialize a list to hold our counts
    n_content = []

    # Determine the length of the longest sequence
    max_length = max(len(seq) for seq in seqs)

    # Loop over each position up to the maximum length
    for i in range(max_length):
        # Initialize a count for 'N' at this position
        n_count = 0
        
        # Loop over each sequence and count the 'N's at this position
        for seq in seqs:
            if i < len(seq) and seq[i] == 'N':
                n_count += 1
        
        # Calculate the percentage of 'N' at this position and add it to our n_content list
        n_content.append(n_count / len(seqs) * 100)

        if n_content[-1] > 20:
            greater_20 += 1
        if n_content[-1] > 10:
            greater_10 += 1

    return n_content, greater_10, greater_20

def plot_n_content(n_content):
    # Create the x-values for our plot (the position in the read)
    x_values = list(range(1, len(n_content)+1))

    # Create a new figure
    plt.figure()

    # Plot the 'N' content
    plt.plot(x_values, n_content, label='N')

    # Add labels and a legend
    plt.xlabel('Position in read (bp)')
    plt.ylabel('N content (%)')
    plt.ylim(0, 100)
    plt.legend()

    # Save and/or display the plot
    plt.savefig("ezqc_output/per_base_N_content_plot.png")
    #plt.show()

def run_pbnc5(seqs):
    content, greater_10, greater_20 = per_base_n_content(seqs)
    plot_n_content(content)
    if (greater_20>0):
        print_color(f"X | Per base N content NOT pass. {greater_20} positions with greater than 20% N content", "red")
        return False
    elif (greater_10):  
        print_color(f"- | Per base N content warning. {greater_10} positions with greater than 10% N content", "yellow")
        return False
    else:
        print_color(f"O | Per base N content pass. No positions with greater than 10% N content", "green")
        return True



import matplotlib.pyplot as plt
from .color_print import print_color
from collections import Counter

def calculate_sequence_lengths(sequences):
    sequence_lengths = [len(seq) for seq in sequences]
    # Detect warning and failure situations
    if 0 in sequence_lengths:
        print_color("X | Some sequences have zero length.","red")
    if len(set(sequence_lengths)) > 1:
        print_color("- | Not all sequences are of the same length.","yellow")
    else:
        print_color("O | Lengths of Sequences are good","green")
        
    return Counter(sequence_lengths)

def plot_sequence_lengths(sequence_lengths):
    plt.figure()
    plt.bar(sequence_lengths.keys(), sequence_lengths.values())
    plt.title("Sequence Length Distribution")
    plt.xlabel("Sequence Length")
    plt.ylabel("Number of Sequences")
    # Save and/or display the plot
    plt.savefig("ezqc_output/Sequence_Length_Distribution.png")
    # plt.show()

def run_sld6(sequences):
    # print("running function 6: Sequence Length Distribution")
    sequence_lengths = calculate_sequence_lengths(sequences)
    plot_sequence_lengths(sequence_lengths)
    return True

    


from Bio.Blast import NCBIWWW
from Bio import SeqIO

for seq in SeqIO.parse('/Users/sky/Desktop/ezqc/tests/SRR020192.fastq', "fastq"):
  result_handle = NCBIWWW.qblast("blastn", "nt", seq.format("fasta"),
                                format_type="Text")
  print(seq)
  output = result_handle.read()
  print(output)
  break


import numpy as np
import matplotlib.pyplot as plt
from .color_print import print_color

def phred33_to_q(qual_string):
    return [ord(ch) - 33 for ch in qual_string]

def mean_qual_score(qual_string):
    qual_scores = phred33_to_q(qual_string)
    return np.mean(qual_scores)

def run_psqs2(quality_strings):
# Calculate mean quality scores for each sequence
    mean_qual_scores = [mean_qual_score(qual_string) for qual_string in quality_strings]

    # Set up bins for the x-axis (mean sequence quality)
    bin_edges = np.arange(0, np.ceil(max(mean_qual_scores)) + 1, 1)

    # Calculate the frequency (counts) for each bin
    counts, _ = np.histogram(mean_qual_scores, bins=bin_edges)

    proportion_low_quality = sum(mqs < 20 for mqs in mean_qual_scores) / len(mean_qual_scores)


    # Create the plot
    plt.figure(figsize=(10, 6))
    plt.bar(bin_edges[:-1], counts, width=1, edgecolor="k", alpha=0.7)
    plt.xlabel("Mean Sequence Quality(Phred scores)")
    plt.ylabel("Number of Sequences")
    plt.title("Quality Scores distribution over all sequences")
    plt.xticks(bin_edges)
    plt.grid(True, linestyle='--', alpha=0.5)

    # Save and/or display the plot
    plt.savefig("ezqc_output/per_sequence_quality_scores.png")
    # plt.show()


    # print(proportion_low_quality)
    if (proportion_low_quality > 0.01):
        print_color(f"X | Per sequence quality score NOT pass. Because proportion of low quality is {100*proportion_low_quality:.2f} %, which is more than 1%","red")
        return False
    else:
        print_color(f"O | Per sequence quality pass. Proportion of low quality is {100*proportion_low_quality:.2f} %, which is less than 1%","green")
        return True


import matplotlib.pyplot as plt
from .color_print import print_color


def per_base_sequence_content(seqs):
    greater_20 = 0
    greater_10 = 0

    # Initialize a dictionary to hold our counts
    base_content = {'A': [], 'T': [], 'C': [], 'G': []}

    # Determine the length of the longest sequence
    max_length = max(len(seq) for seq in seqs)

    # Loop over each position up to the maximum length
    for i in range(max_length):
        # Initialize a dictionary to hold the counts for this position
        pos_counts = {'A': 0, 'T': 0, 'C': 0, 'G': 0, 'N': 0}
        
        # Loop over each sequence and count the bases at this position
        for seq in seqs:
            if i < len(seq):
                pos_counts[seq[i]] += 1
        
        # Calculate the percentage of each base at this position and add it to our base_content dictionary
        total_count = sum(pos_counts.values())
        for base in base_content.keys():
            base_content[base].append(pos_counts[base] / total_count * 100)
        
        if abs(base_content['A'][-1] - base_content['T'][-1]) > 20 or abs(base_content['G'][-1] - base_content['C'][-1]) > 20:
            greater_20 += 1
        if abs(base_content['A'][-1] - base_content['T'][-1]) > 10 or abs(base_content['G'][-1] - base_content['C'][-1]) > 10:
            greater_10 += 1

    return base_content, greater_10, greater_20



def plot_base_content(base_content):
    # Create the x-values for our plot (the position in the read)
    x_values = list(range(1, len(base_content['A'])+1))

    plt.figure()

    # Plot the base content for each base 
    for base, y_values in base_content.items():
        plt.plot(x_values, y_values, label=base)

    # Add labels and a legend
    plt.xlabel('Position in read (bp)')
    plt.ylabel('Base content (%)')
    plt.ylim(0, 100)
    plt.legend()

    # Save and/or display the plot
    plt.savefig("ezqc_output/per_base_sequence_content_plot.png")
    #plt.show()

def run_pbsc3(seqs):
    content, greater_10, greater_20 = per_base_sequence_content(seqs)
    plot_base_content(content)
    if (greater_20>0):
        print_color(f"X | Per base sequence content NOT pass. {greater_20} positions with greater than 20% differences", "red")
        return False
    elif (greater_10):  
        print_color(f"X | Per base sequence content warning. {greater_10} positions with greater than 10% differences", "yellow")
        return False
    else:
        print_color(f"X | Per base sequence content pass. No positions with greater than 10% differences", "green")
        return True



import collections
import csv
from .color_print import print_color

def run_os7(dna_sequences):
    total_count = len(dna_sequences)
    
    counter = collections.Counter(dna_sequences)
    
    table = []
    
    sequences_exceeding_1_percent = 0
    sequences_exceeding_0_1_percent = 0

    for sequence, count in counter.items():
        percentage = (count / total_count) * 100

        if percentage > 1:
            sequences_exceeding_1_percent += 1
            table.append([sequence, count, percentage])

        elif percentage > 0.1:
            sequences_exceeding_0_1_percent += 1
            table.append([sequence, count, percentage])

    with open('ezqc_output/overrepresented_sequences.csv', 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerow(["Sequence", "Count", "Percentage"])
        writer.writerows(table)

    if (sequences_exceeding_1_percent > 0):
        print_color(f"X | Overrepresented sequences NOT pass. {sequences_exceeding_1_percent} sequences representing more than 1% of the total", "red")
        return False
    elif (sequences_exceeding_0_1_percent > 0):
        print_color(f"- | Overrepresented sequences warning. {sequences_exceeding_0_1_percent} sequences representing more than 0.1% of the total", "yellow")
        return False
    else:
        print_color(f"O | Overrepresented sequences pass. No sequences representing more than 0.1% of the total", "green")
        return True



import pytest
import argparse
import os
import numpy as np
from ezqc.pbsq1 import run_pbsq1
from ezqc.psqs2 import run_psqs2
from ezqc.pbsc3 import run_pbsc3
from ezqc.psgc4 import run_psgc4
from ezqc.pbnc5 import run_pbnc5
from ezqc.sld6 import run_sld6
from ezqc.os7 import run_os7
from ezqc.ac8 import run_ac8


def test_addition():
    assert 2 + 2 == 4


# with open ("tests/SRR020192.fastq",'r') as file_obj:
#     header = file_obj.readline().strip()
#     if not header:
#         break
#     sequence = file_obj.readline().strip()
#     file_obj.readline()
#     quality_str = file_obj.readline().strip()
#     yield header, sequence, quality_str

# file.close()

# # Create the argparse object and define the arguments
# parser = argparse.ArgumentParser(description="EZQC FastQ Quality Analyzer")
# parser.add_argument('seqs', metavar='SEQ', nargs='+', help='input .fastq file(s)')

# # Parse the command-line arguments 
# args = parser.parse_args()

# directory_name = "ezqc_output"
# # Get the current working directory
# current_directory = os.getcwd()
# # Create a path for the new directory
# new_directory_path = os.path.join(current_directory, directory_name)

# # Create the directory if it doesn't already exist
# if not os.path.exists(new_directory_path):
#     os.makedirs(new_directory_path)
#     print(f"Directory '{directory_name}' created successfully!")
# else:
#     print(f"Directory '{directory_name}' already exists.")

# total_num = 0
# pass_num = 0

# # Process each .fastq file
# for file_path in args.seqs:
#     print(f"Processing file: {file_path}")
#     total_num += 1
#     headers = []
#     sequences = []
#     quality_strings = []

#     with open(file_path, 'r') as fastq_file:
#         for header, sequence, quality_str in parse_fastq_file(fastq_file):
#             headers.append(header)
#             sequences.append(sequence)
#             quality_strings.append(quality_str)
    # average_length_each_sequence = int(np.average([len(x) for x in quality_strings]))

# def function1():
    
# def function2():
    
# def function3():
    
# def function4():
    


# Color escape sequences
RESET = "\033[0m"
BOLD = "\033[1m"
UNDERLINE = "\033[4m"
# Text color escape sequences
RED = "\033[31m"
GREEN = "\033[32m"
YELLOW = "\033[33m"
BLUE = "\033[34m"
PURPLE = "\033[35m"
CYAN = "\033[36m"

def print_color(text, color):
    if (color == "yellow" or color == "YELLOW"):
        print(YELLOW + text + RESET)
    if (color == "green" or color == "GREEN"):
        print(GREEN + text + RESET)
    if (color == "RED" or color == "red"):
        print(RED + text + RESET)
    



import argparse
import numpy as np
from .pbsq1 import run_pbsq1
from .psqs2 import run_psqs2
from .pbsc3 import run_pbsc3
from .psgc4 import run_psgc4
from .pbnc5 import run_pbnc5
from .sld6 import run_sld6
from .os7 import run_os7
from .ac8 import run_ac8
import os

def parse_fastq_file(file_obj):
    while True:
        header = file_obj.readline().strip()
        if not header:
            break
        sequence = file_obj.readline().strip()
        file_obj.readline()
        quality_str = file_obj.readline().strip()
        yield header, sequence, quality_str

def main():
    logo = r'''
    ███████╗███████╗ ██████╗  ██████╗
    ██╔════╝╚══███╔╝██╔═══██╗██╔════╝
    █████╗    ███╔╝ ██║   ██║██║     
    ██╔══╝   ███╔╝  ██║▄▄ ██║██║     
    ███████╗███████╗╚██████╔╝╚██████╗
    ╚══════╝╚══════╝ ╚══▀▀═╝  ╚═════╝ made by Tinger Shi & Sky Li
    '''
    
    print(logo)

    # Create the argparse object and define the arguments
    parser = argparse.ArgumentParser(description="EZQC FastQ Quality Analyzer")
    parser.add_argument('seqs', metavar='SEQ', nargs='+', help='input .fastq file(s)')

    # Parse the command-line arguments 
    args = parser.parse_args()

    directory_name = "ezqc_output"
    # Get the current working directory
    current_directory = os.getcwd()
    # Create a path for the new directory
    new_directory_path = os.path.join(current_directory, directory_name)

    # Create the directory if it doesn't already exist
    if not os.path.exists(new_directory_path):
        os.makedirs(new_directory_path)
        print(f"Directory '{directory_name}' created successfully!")
    else:
        print(f"Directory '{directory_name}' already exists.")

    total_num = 0
    pass_num = 0

    # Process each .fastq file
    for file_path in args.seqs:
        print(f"Processing file: {file_path}")
        total_num += 1
        headers = []
        sequences = []
        quality_strings = []

        with open(file_path, 'r') as fastq_file:
            for header, sequence, quality_str in parse_fastq_file(fastq_file):
                headers.append(header)
                sequences.append(sequence)
                quality_strings.append(quality_str)
        # average_length_each_sequence = int(np.average([len(x) for x in quality_strings]))

    results = [
        run_pbsq1(quality_strings),
        run_psqs2(quality_strings),
        run_pbsc3(sequences),
        run_psgc4(sequences),
        run_pbnc5(sequences),
        run_sld6(sequences),
        run_os7(sequences),
        run_ac8(sequences)
    ]
    if all(results): 
        pass_num += 1
    print(f"All fastq file analyzied, {pass_num}/{total_num} passed all tests.")

if __name__ == '__main__':
    main()



import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import norm
from .color_print import print_color
# need to pip install scipy

def detect_warnings_failures(gc_contents,sequences):
    mu, std = norm.fit(gc_contents)
    deviations = [abs(content - mu) for content in gc_contents]

    sum_deviation = sum(deviations)

    warning_threshold = 0.15 * len(sequences)
    failure_threshold = 0.30 * len(sequences)

    if sum_deviation > warning_threshold and sum_deviation <= failure_threshold:
        print_color("- | The sum of the deviations from the normal distribution represents more than 15% of the reads.","yellow")
        return False
    elif sum_deviation > failure_threshold:
        print_color("X | The sum of the deviations from the normal distribution represents more than 30% of the reads.","red")
        return False
    else:
        print_color("O | The sum of the deviations from the normal distribution are good","green")
        return True

def run_psgc4(sequences):
    gc_contents = []
    # Iterate through each sequence
    for seq in sequences:
        # Compute the GC content for the current sequence
        count_gc = sum(base in "GC" for base in seq)
        gc_percent = 100 * count_gc / len(seq)

        # Append the GC content to the list
        gc_contents.append(gc_percent)
        
    # plot GC distribution over all sequences
    plt.figure(figsize=(10,6))
    plt.hist(gc_contents, bins=100, alpha=0.5, label='Observed GC')

    # calculate and plot theoretical distribution
    mu, sigma = np.mean(gc_contents), np.std(gc_contents)
    s = np.arange(mu - 3.5*sigma, mu + 3.5*sigma, 0.05)
    plt.plot(s, norm.pdf(s, mu, sigma)*len(gc_contents)*(max(gc_contents)-min(gc_contents))/100, label='Theoretical GC')

    plt.title("GC distribution over all sequences")
    plt.xlabel("Mean GC content")
    plt.ylabel("Number of sequences")
    plt.legend(loc='upper right')
    plt.grid(True)
    # Save and/or display the all the plots
    plt.savefig("ezqc_output/per_sequence_GC_content.png")
    # plt.show()

    return detect_warnings_failures(gc_contents,sequences)


from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ezqc',
    version='0.2',
    author='Tinger Shi & Sky Li',
    description='EZQC is a streamlined, terminal-based alternative to FastQC.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        # Add your project dependencies here
        'numpy',
        'matplotlib',
        'pandas',
        'scipy',
        'Bio',
    ],
    entry_points={
        'console_scripts': [
            'ezqc=ezqc.ezqc:main',
        ],
    },
)



import numpy as np
import matplotlib.pyplot as plt
from .color_print import print_color

threshold = 24

def find_largest_range(numbers):
    start = end = max_start = max_end = 0
    max_length = 0

    for i in range(1, len(numbers)):
        if numbers[i] == numbers[i-1] + 1:
            end = i
        else:
            length = end - start + 1
            if length > max_length:
                max_length = length
                max_start = start
                max_end = end
            start = end = i

    # Check if the last range is the largest
    length = end - start + 1
    if length > max_length:
        max_length = length
        max_start = start
        max_end = end

    return numbers[max_start], numbers[max_end]


def phred33_to_q(quality_str):
    return np.array([ord(ch) - 33 for ch in quality_str])

def calculate_quality_scores(quality_strings):
    num_sequences = len(quality_strings)
    max_length = max(len(qstr) for qstr in quality_strings)
    quality_scores = np.zeros((num_sequences, max_length))
    mask = np.zeros_like(quality_scores, dtype=bool)

    for i, qstr in enumerate(quality_strings):
        scores = phred33_to_q(qstr)
        quality_scores[i, :len(scores)] = scores
        mask[i, :len(scores)] = True
    
    return quality_scores, mask

def run_pbsq1(quality_strings):
    # Convert quality strings to quality scores 
    quality_scores, mask = calculate_quality_scores(quality_strings)

    # Calculate the number of reads and the maximum length of the reads
    max_read_length = quality_scores.shape[1]

    # Calculate average, lower quartile, and upper quartile quality scores for each base position
    masked_quality_scores = np.ma.masked_array(quality_scores, ~mask)
    average_quality_scores = masked_quality_scores.mean(axis=0).data
    lower_quartile_scores = np.ma.apply_along_axis(lambda x: np.percentile(x.compressed(), 25), 0, masked_quality_scores)
    upper_quartile_scores = np.ma.apply_along_axis(lambda x: np.percentile(x.compressed(), 75), 0, masked_quality_scores)

    # Create the plot
    positions = np.arange(1, max_read_length+1)
    plt.plot(positions, average_quality_scores, label='Mean')
    plt.plot(positions, lower_quartile_scores, label='Lower quartile', linestyle='--')
    plt.plot(positions, upper_quartile_scores, label='Upper quartile', linestyle='--')

    plt.xlabel('Position in read (bp)')
    plt.ylabel('Quality score')
    plt.title('Per base sequence quality plot')
    plt.xticks(np.arange(1, max_read_length+1, max(1, max_read_length//15)))  # Adjust x-axis tick spacing
    plt.yticks(np.arange(0, np.max(average_quality_scores)+1, 2))
    plt.grid(True)
    plt.legend()
    plt.savefig("ezqc_output/per_base_sequence_quality_plot.png")
    # plt.show()

    avg_score = np.average(average_quality_scores)

    if avg_score < threshold*0.75:
        print_color(f"X | Per base sequence quality NOT pass. Low average quality score of {avg_score:.2f}", "green")
        return False
    elif np.all(average_quality_scores >= threshold):
        print_color(f"O | Per base sequence quality pass. With high average quality score of {avg_score:.2f}", "red")
        return True
    else:
        indices_above_threshold = np.where(average_quality_scores >= threshold)[0]
        start, end = find_largest_range(indices_above_threshold)
        print_color(f"- | Per base sequence quality can be improved. With high quality reads from position {start} to {end}", "yellow")
        return False



import matplotlib.pyplot as plt
from .color_print import print_color

adapters = {
    'Illumina Universal Adapter': 'AGATCGGAAGAG',
    'Illumina Small RNA 3\' Adapter': 'TGGAATTCTCGG',
    'Illumina Small RNA 5\' Adapter': 'GATCGTCGGACT',
    'SOLiD Small RNA Adapter': 'CTGCTGTACGGCCAAGGCG'
}

def calculate_adapter_content(sequences, adapters):

    counts = {name: [0]*len(seq) for name, seq in adapters.items()}

    for seq in sequences:
        for name, adapter in adapters.items():
            for i in range(len(seq) - len(adapter) + 1):
                if seq[i:i+len(adapter)] == adapter:
                    counts[name][i] += 1

    percentages = {name: [count/len(sequences)*100 for count in count_list] for name, count_list in counts.items()}

    all_pass = True
    for name, percentage in percentages.items():
        if max(percentage) > 10:
            print_color(f'X | {name} is present in more than 10% of all reads','red')
            all_pass = False
        elif (max(percentage) > 5):
            print_color(f'- | {name} is present in more than 5% of all reads','yellow')
            all_pass = False
    if (all_pass):
        print_color(f'O | No adaptor is present in more than 5% of all reads','green')

    return percentages

def plot_adapter_content(adapter_percentages):
    plt.figure()
    positions = list(range(1, max(map(len, adapter_percentages.values())) + 1))

    for name, percentages in adapter_percentages.items():
        plt.plot(positions[:len(percentages)], percentages, label=name)

    plt.xlabel('Position in read (bp)')
    plt.ylabel('% Adapter')
    plt.ylim(0, 100)
    plt.title('Adapter Content')
    plt.legend()

    # Save and/or display the plot
    plt.savefig("ezqc_output/adaptor_content_plot.png")
    #plt.show()

def run_ac8(seqs):
    adapter_percentages = calculate_adapter_content(seqs, adapters)
    plot_adapter_content(adapter_percentages)